@@ -37,6 +37,9 @@ log = logging.getLogger("braille")
 import signal
 import os
 
+import gobject
+gobject.threads_init()
+
 try:
     import louis
 except ImportError:
@@ -44,8 +47,6 @@ except ImportError:
 
 try:
     import brlapi
-    import gobject
-    gobject.threads_init()
 
     _brlAPI = None
     _brlAPIAvailable = True
@@ -76,6 +77,13 @@ from orca_i18n import _                          # for gettext support
 #
 _monitor = None
 
+# brlapi keys which are not allowed to interupt speech:
+#
+dontInteruptSpeechKeys = []
+if _brlAPIAvailable:
+    dontInteruptSpeechKeys = [brlapi.KEY_CMD_FWINLT, brlapi.KEY_CMD_FWINRT, \
+        brlapi.KEY_CMD_LNUP, brlapi.KEY_CMD_LNDN]
+
 # Common names for most used BrlTTY commands, to be shown in the GUI:
 # ATM, the ones used in default.py are:
 #
@@ -187,6 +195,15 @@ _lines = []
 #
 _regionWithFocus = None
 
+# The last text information painted.  This has the following fields:
+#
+# lastTextObj = the last accessible
+# lastCaretOffset = the last caret offset of the last text displayed
+# lastLineOffset = the last line offset of the last text displayed
+# lastCursorCell = the last cell on the braille display for the caret
+#
+_lastTextInfo = (None, 0, 0, 0)
+
 # The viewport is a rectangular region of size _displaySize whose upper left
 # corner is defined by the point (x, line number).  As such, the viewport is
 # identified solely by its upper left point.
@@ -209,6 +226,14 @@ beginningIsShowing = False
 #
 cursorCell = 0
 
+# The event source of a timeout used for flashing a message.
+#
+_flashEventSourceId = 0
+
+# Line information saved prior to flashing any messages
+#
+_saved = None
+
 # Translators: These are the braille translation table names for different
 # languages. You could read about braille tables at:
 # http://en.wikipedia.org/wiki/Braille
@@ -1047,7 +1072,128 @@ def setFocus(region, panToFocus=True, getLinkMask=True):
 
     viewport[0] = max(0, offset)
 
-def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True):
+def _realignViewport(string, focusOffset, cursorOffset):
+    """Realigns the braille display to account for braille alignment
+    preferences.  By the time this method is called, if there is a
+    cursor cell to be displayed, it should already be somewhere in
+    the viewport.  All we're going to do is adjust the viewport a
+    little to align the viewport edge according to the
+    settings.brailleAlignmentStyle.
+
+    Arguments:
+    - string: the entire string to be presented
+    - focusOffset: where in string the focused region begins
+    - cursorOffset: where in the string the cursor should be
+
+    Returns: the viewport[0] value is potentially modified.
+    """
+
+    # pylint complains we don't set viewport, which in fact we do if
+    # 'jump' ends up being set.
+    #
+    # pylint: disable-msg=W0602
+    #
+    global viewport
+
+    jump = 0
+
+    # If there's no cursor to show or we're doing
+    # ALIGN_BRAILLE_BY_EDGE, the viewport should already be where it
+    # belongs.  Otherwise, we may need to do some adjusting of the
+    # viewport.
+    #
+    if (cursorOffset < 0) \
+       or (settings.brailleAlignmentStyle == settings.ALIGN_BRAILLE_BY_EDGE):
+        pass
+    else:
+        # The left and right margin values are absolute values in the
+        # string and represent where in the string the margins of the
+        # current viewport lie.  Note these are margins and not the
+        # actual edges of the viewport.
+        #
+        leftMargin = viewport[0] + settings.brailleAlignmentMargin
+        rightMargin = (viewport[0] + _displaySize[0]) \
+                      - settings.brailleAlignmentMargin
+
+        # This represents how far left in the string we want to search
+        # and also how far left we'll realign the viewport. Setting it
+        # to focusOffset means we won't move the viewport further left
+        # than the beginning of the current region with focus.
+        #
+        leftMostEdge = max(0, focusOffset)
+
+        # If we align by margin, we just want to keep the cursor at or
+        # in between the margins.  The only time we go outside the
+        # margins are when we are at the ends of the string.
+        #
+        if settings.brailleAlignmentStyle == settings.ALIGN_BRAILLE_BY_MARGIN:
+            if cursorOffset < leftMargin:
+                jump = cursorOffset - leftMargin
+            elif cursorOffset > rightMargin:
+                jump = cursorOffset - rightMargin
+        elif settings.brailleAlignmentStyle == settings.ALIGN_BRAILLE_BY_WORD:
+            # When we align by word, we want to try to show complete
+            # words at the edges of the braille display.  When we're
+            # near the left edge, we'll try to start a word at the
+            # left edge.  When we're near the right edge, we'll try to
+            # end a word at the right edge.
+            #
+            if cursorOffset < leftMargin:
+                # Find the index of the character that is the first
+                # letter of the word prior to left edge of the
+                # viewport.
+                #
+                inWord = False
+                leftWordEdge = viewport[0] - 1
+                while leftWordEdge >= leftMostEdge:
+                    if not string[leftWordEdge] in ' \t\n\r\v\f':
+                        inWord = True
+                    elif inWord:
+                        leftWordEdge += 1
+                        break
+                    leftWordEdge -= 1
+                leftWordEdge = max(leftMostEdge, leftWordEdge)
+                jump = leftWordEdge - viewport[0]
+            elif cursorOffset > rightMargin:
+                # Find the index of the character that is the last
+                # letter of the word after the right edge of the
+                # viewport.
+                #
+                inWord = False
+                rightWordEdge = viewport[0] + _displaySize[0]
+                while rightWordEdge < len(string):
+                    if not string[rightWordEdge] in ' \t\n\r\v\f':
+                        inWord = True
+                    elif inWord:
+                        break
+                    rightWordEdge += 1
+                rightWordEdge = min(len(string), rightWordEdge)
+                jump = max(0, rightWordEdge - (viewport[0] + _displaySize[0]))
+
+            # We use the brailleMaximumJump to help us handle really
+            # long words.  The (jump/abs(jump)) stuff is a quick and
+            # dirty way to retain the sign (i.e., +1 or -1).
+            #
+            if abs(jump) > settings.brailleMaximumJump:
+                jump = settings.brailleMaximumJump * (jump/abs(jump))
+
+    if jump:
+        # Set the viewport's left edge based upon the jump, making
+        # sure we don't go any farther left than the leftMostEdge.
+        #
+        viewport[0] = max(leftMostEdge, viewport[0] + jump)
+
+        # Now, make sure we don't scroll too far to the right.  That
+        # is, avoid showing blank spaces to the right if there is more
+        # of the string that can be shown.
+        #
+        viewport[0] = min(viewport[0],
+                          max(leftMostEdge, len(string) - _displaySize[0]))
+
+def refresh(panToCursor=True,
+            targetCursorCell=0,
+            getLinkMask=True,
+            stopFlash=True):
     """Repaints the Braille on the physical display.  This clips the entire
     logical structure by the viewport and also sets the cursor to the
     appropriate location.  [[[TODO: WDW - I'm not sure how BrlTTY handles
@@ -1068,12 +1214,31 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True):
       attributeMask for links. Reasons we might not want to include
       knowning that we will fail and/or it taking an unreasonable
       amount of time (AKA Gecko).
+    - stopFlash: if True, kill any flashed message that may be showing.
     """
 
     global endIsShowing
     global beginningIsShowing
     global cursorCell
     global _monitor
+    global _lastTextInfo
+
+    # Check out what we were displaying the last time - it might be
+    # the same text object we are displaying now.
+    #
+    (lastTextObj, lastCaretOffset, lastLineOffset, lastCursorCell) = \
+        _lastTextInfo
+    if _regionWithFocus and isinstance(_regionWithFocus, Text):
+        currentTextObj = _regionWithFocus.accessible
+        currentCaretOffset = _regionWithFocus.caretOffset
+        currentLineOffset = _regionWithFocus.lineOffset
+    else:
+        currentTextObj = None
+        currentCaretOffset = 0
+        currentLineOffset = 0
+
+    if stopFlash:
+        killFlash(restoreSaved=False)
 
     if len(_lines) == 0:
         if not _brlAPIRunning:
@@ -1086,6 +1251,7 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True):
                               "BrlTTY seems to have disappeared:")
                 debug.printException(debug.LEVEL_WARNING)
                 shutdown()
+        _lastTextInfo = (None, 0, 0, 0)
         return
 
     # Now determine the location of the cursor.  First, we'll figure
@@ -1096,6 +1262,26 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True):
     if targetCursorCell < 0:
         targetCursorCell = _displaySize[0] + targetCursorCell + 1
 
+    # If there is no target cursor cell, then try to set one.  We
+    # currently only do this for text objects, and we do so by looking
+    # at the last position of the caret offset and cursor cell.  The
+    # primary goal here is to keep the cursor movement on the display
+    # somewhat predictable.
+    #
+    if (targetCursorCell == 0) \
+       and currentTextObj and (currentTextObj == lastTextObj) \
+       and (currentLineOffset == lastLineOffset):
+        if lastCaretOffset == currentCaretOffset:
+            targetCursorCell = lastCursorCell
+        elif lastCaretOffset < currentCaretOffset:
+            targetCursorCell = min(_displaySize[0],
+                                   lastCursorCell \
+                                   + (currentCaretOffset - lastCaretOffset))
+        elif lastCaretOffset > currentCaretOffset:
+            targetCursorCell = max(1,
+                                   lastCursorCell \
+                                   - (lastCaretOffset - currentCaretOffset))
+
     # Now, we figure out the 0-based offset for where the cursor
     # actually is in the string.
     #
@@ -1120,6 +1306,12 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True):
         elif cursorOffset >= (viewport[0] + _displaySize[0]):
             viewport[0] = max(0, cursorOffset - _displaySize[0] + 1)
 
+    # The cursorOffset should be somewhere in the viewport right now.
+    # Let's try to realign the viewport so that the cursor shows up
+    # according to the settings.brailleAlignmentStyle setting.
+    #
+    _realignViewport(string, focusOffset, cursorOffset)
+
     startPos = viewport[0]
     endPos = startPos + _displaySize[0]
 
@@ -1200,7 +1392,77 @@ def refresh(panToCursor=True, targetCursorCell=0, getLinkMask=True):
     beginningIsShowing = startPos == 0
     endIsShowing = endPos >= len(string)
 
-def displayRegions(regionInfo):
+    # Remember the text information we were presenting (if any)
+    #
+    if _regionWithFocus and isinstance(_regionWithFocus, Text):
+        _lastTextInfo = (_regionWithFocus.accessible,
+                         _regionWithFocus.caretOffset,
+                         _regionWithFocus.lineOffset,
+                         cursorCell)
+    else:
+        _lastTextInfo = (None, 0, 0, 0)
+
+def _flashCallback():
+    global _lines
+    global _regionWithFocus
+    global viewport
+    global _flashEventSourceId
+
+    if _flashEventSourceId:
+        (_lines, _regionWithFocus, viewport, flashTime) = _saved
+        refresh(panToCursor=False, stopFlash=False)
+        _flashEventSourceId = 0
+
+    return False
+
+def killFlash(restoreSaved=True):
+    global _flashEventSourceId
+    global _lines
+    global _regionWithFocus
+    global viewport
+    if _flashEventSourceId:
+        if _flashEventSourceId > 0:
+            gobject.source_remove(_flashEventSourceId)
+        if restoreSaved:
+            (_lines, _regionWithFocus, viewport, flashTime) = _saved
+            refresh(panToCursor=False, stopFlash=False)
+        _flashEventSourceId = 0
+
+def resetFlashTimer():
+    global _flashEventSourceId
+    if _flashEventSourceId > 0:
+        gobject.source_remove(_flashEventSourceId)
+        flashTime = _saved[3]
+        _flashEventSourceId = gobject.timeout_add(flashTime, _flashCallback)
+
+def _initFlash(flashTime):
+    """Sets up the state needed to flash a message or clears any existing
+    flash if nothing is to be flashed.
+
+    Arguments:
+    - flashTime:  if non-0, the number of milliseconds to display the
+                  regions before reverting back to what was there before.
+                  A 0 means to not do any flashing.  A negative number
+                  means display the message until some other message
+                  comes along or the user presses a cursor routing key.
+    """
+
+    global _saved
+    global _flashEventSourceId
+
+    if _flashEventSourceId:
+        if _flashEventSourceId > 0:
+            gobject.source_remove(_flashEventSourceId)
+        _flashEventSourceId = 0
+    else:
+        _saved = (_lines, _regionWithFocus, viewport, flashTime)
+
+    if flashTime > 0:
+        _flashEventSourceId = gobject.timeout_add(flashTime, _flashCallback)
+    elif flashTime < 0:
+        _flashEventSourceId = -666
+
+def displayRegions(regionInfo, flashTime=0):
     """Displays a list of regions on a single line, setting focus to the
        specified region.  The regionInfo parameter is something that is
        typically returned by a call to braille_generator.generateBraille.
@@ -1209,8 +1471,14 @@ def displayRegions(regionInfo):
     - regionInfo: a list where the first element is a list of regions
                   to display and the second element is the region
                   with focus (must be in the list from element 0)
+    - flashTime:  if non-0, the number of milliseconds to display the
+                  regions before reverting back to what was there before.
+                  A 0 means to not do any flashing.  A negative number
+                  means display the message until some other message
+                  comes along or the user presses a cursor routing key.
     """
 
+    _initFlash(flashTime)
     regions = regionInfo[0]
     focusedRegion = regionInfo[1]
 
@@ -1220,22 +1488,28 @@ def displayRegions(regionInfo):
         line.addRegion(item)
     addLine(line)
     setFocus(focusedRegion)
-    refresh()
+    refresh(stopFlash=False)
 
-def displayMessage(message, cursor=-1):
+def displayMessage(message, cursor=-1, flashTime=0):
     """Displays a single line, setting the cursor to the given position,
     ensuring that the cursor is in view.
 
     Arguments:
     - message: the string to display
     - cursor: the 0-based cursor position, where -1 (default) means no cursor
+    - flashTime:  if non-0, the number of milliseconds to display the
+                  regions before reverting back to what was there before.
+                  A 0 means to not do any flashing.  A negative number
+                  means display the message until some other message
+                  comes along or the user presses a cursor routing key.
     """
 
+    _initFlash(flashTime)
     clear()
     region = Region(message, cursor)
     addLine(Line(region))
     setFocus(region)
-    refresh(True)
+    refresh(True, stopFlash=False)
 
 def panLeft(panAmount=0):
     """Pans the display to the left, limiting the pan to the beginning
@@ -1335,6 +1609,12 @@ def processRoutingKey(event):
     the dictionary form of the expanded BrlAPI event.
     """
 
+    # If a message is being flashed, we'll use a routing key to dismiss it.
+    #
+    if _flashEventSourceId:
+        killFlash()
+        return
+
     cell = event.event["argument"]
 
     if len(_lines) > 0:
