@@ -19,9 +19,9 @@
 
 """Provides a SpeechServer factory for gnome-speech drivers."""
 
-__id__        = "$Id: gnomespeechfactory.py 4453 2009-01-22 00:35:18Z wwalker $"
-__version__   = "$Revision: 4453 $"
-__date__      = "$Date: 2009-01-21 19:35:18 -0500 (Wed, 21 Jan 2009) $"
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -36,6 +36,19 @@ import time
 import bonobo
 import ORBit
 
+# Init the ORB if we need to.  With AT-SPI/CORBA, we depend upon the pyatspi
+# implementation to init the ORB for us.  [[[WDW: With AT-SPI/D-Bus, we need
+# to do it ourselves.  I'm not sure how to ask ORBit itself if it's been
+# init'ed yet, so I do so indirectly by looking for an attribute of pyatspi.
+# This attribute has been set if AT-SPI/CORBA is in use and it not set if
+# AT-SPI/D-Bus is in use.]]]
+#
+try:
+    import pyatspi
+    poa = pyatspi.Accessibility__POA
+except AttributeError:
+    orb = ORBit.CORBA.ORB_init()
+
 import chnames
 import debug
 import orca
@@ -111,7 +124,7 @@ class _Speaker(GNOME__POA.Speech.SpeechCallback):
     all calls to the 'real' gnome speech speaker.
     """
 
-    def __init__(self, gnome_speaker):
+    def __init__(self, gnome_speaker, voice):
 
         # We know what we are doing here, so tell pylint not to flag
         # the self._this() method call as an error.  The disable-msg is
@@ -123,6 +136,7 @@ class _Speaker(GNOME__POA.Speech.SpeechCallback):
         if settings.enableSpeechCallbacks:
             gnome_speaker.registerSpeechCallback(self._this())
         self.__callbacks = []
+        self.voiceInfo = [voice.name, voice.language, voice.gender]
 
     def registerCallback(self, callback):
         self.__callbacks.append(callback)
@@ -504,7 +518,7 @@ class SpeechServer(speechserver.SpeechServer):
         #
         # pylint: disable-msg=W0212
 
-        speaker = _Speaker(s._narrow(GNOME.Speech.Speaker))
+        speaker = _Speaker(s._narrow(GNOME.Speech.Speaker), voice)
 
         # Turn off punctuation.  We do this because we want to handle
         # spoken punctuation on our own.  Only do so if "punctuation mode"
@@ -521,17 +535,46 @@ class SpeechServer(speechserver.SpeechServer):
 
         speaker.registerCallback(self)
 
-        parameters = speaker.getSupportedParameters()
-        for parameter in parameters:
-            if parameter.name == "rate":
-                self.__rateInfo[speaker] = \
-                    [parameter.min, parameter.current, parameter.max]
-            elif parameter.name == "pitch":
-                self.__pitchInfo[speaker] = \
-                    [parameter.min, parameter.current, parameter.max]
-            elif parameter.name == "volume":
-                self.__volumeInfo[speaker] = \
-                    [parameter.min, parameter.current, parameter.max]
+        # Hack - see if we have created a new speaker for a new voice
+        # for the engine.  If so, save away the min/current/max parameters.
+        # Otherwise, use the ones we saved away.  The problem is that
+        # each time we get the parameters, the current one reflects the
+        # current value.  We only want to save the value the first time
+        # because at that time, the current value is the default value.
+        #
+        saveParameters = True
+        for existingSpeaker in self.__speakers.values():
+            if existingSpeaker.voiceInfo == speaker.voiceInfo:
+                try:
+                    self.__rateInfo[speaker] = \
+                        self.__rateInfo[existingSpeaker]
+                except:
+                    pass
+                try:
+                    self.__pitchInfo[speaker] = \
+                        self.__pitchInfo[existingSpeaker]
+                except:
+                    pass
+                try:
+                    self.__volumeInfo[speaker] = \
+                        self.__volumeInfo[existingSpeaker]
+                except:
+                    pass
+                saveParameters = False
+                break
+
+        if saveParameters:
+            parameters = speaker.getSupportedParameters()
+            for parameter in parameters:
+                if parameter.name == "rate":
+                    self.__rateInfo[speaker] = \
+                        [parameter.min, parameter.current, parameter.max]
+                elif parameter.name == "pitch":
+                    self.__pitchInfo[speaker] = \
+                        [parameter.min, parameter.current, parameter.max]
+                elif parameter.name == "volume":
+                    self.__volumeInfo[speaker] = \
+                        [parameter.min, parameter.current, parameter.max]
 
         if ACSS.RATE in acss:
             self.__setRate(speaker, acss[ACSS.RATE])
@@ -935,7 +978,7 @@ class SpeechServer(speechserver.SpeechServer):
         if self.__sayAll:
             self.stop()
 
-        if not acss and text and text.isupper():
+        if not acss and text and text.decode("UTF-8").isupper():
             try:
                 acss = \
                     orca_state.activeScript.voices[settings.UPPERCASE_VOICE]
