@@ -595,11 +595,16 @@ def keyEcho(event):
 
             modifiers = event.modifiers
 
+            brailleMessage = None
             if event_string == "Caps_Lock":
                 if modifiers & (1 << pyatspi.MODIFIER_SHIFTLOCK):
                     eventType = KeyEventType.LOCKING_UNLOCKED
+                    brailleMessage = keynames.getKeyName(event_string) \
+                                     + " " + _("off")
                 else:
                     eventType = KeyEventType.LOCKING_LOCKED
+                    brailleMessage = keynames.getKeyName(event_string) \
+                                     + " " + _("on")
 
             elif event_string == "Num_Lock":
                 # [[[TODO: richb - we are not getting a correct modifier
@@ -613,6 +618,10 @@ def keyEcho(event):
                 #    eventType = KeyEventType.LOCKING_LOCKED
                 pass
 
+            if brailleMessage:
+                braille.displayMessage(brailleMessage,
+                                       flashTime=settings.brailleFlashTime)
+
         elif isFunctionKey(event_string):
             if not settings.enableFunctionKeys:
                 return False
@@ -759,6 +768,9 @@ def _processKeyboardEvent(event):
 
     keyboardEvent = KeyboardEvent(event)
 
+    if keyboardEvent.type == pyatspi.KEY_PRESSED_EVENT:
+        braille.killFlash()
+
     # See if this is one of our special Orca modifier keys.
     #
     # Just looking at the keycode should suffice, but there is a
@@ -897,9 +909,9 @@ def _processBrailleEvent(event):
 
     # Braille key presses always interrupt speech.
     #
-    speech.stop()
-
     event = BrailleEvent(event)
+    if event.event['command'] not in braille.dontInteruptSpeechKeys:
+        speech.stop()
     orca_state.lastInputEvent = event
 
     try:
