@@ -339,7 +339,7 @@ class Script(default.Script):
         speech.speak(utterances)
         settings.speechVerbosityLevel = savedSpeechVerbosityLevel
 
-        braille.displayRegions(brailleGen.getBrailleRegions(tab))
+        braille.displayRegions(brailleGen.generateBraille(tab))
 
     def getTimeForCalRow(self, row, noIncs):
         """Return a string equivalent to the time of the given row in
@@ -577,7 +577,7 @@ class Script(default.Script):
                     if mode == pyatspi.TEXT_BOUNDARY_LINE_START or \
                        len(mystr) == 0 or mystr[len(mystr)-1] in '.?!':
                         string = self.adjustForRepeats(string)
-                        if string.isupper():
+                        if string.decode("UTF-8").isupper():
                             voice = settings.voices[settings.UPPERCASE_VOICE]
                         else:
                             voice = settings.voices[settings.DEFAULT_VOICE]
@@ -606,7 +606,7 @@ class Script(default.Script):
         #
         if len(string) != 0:
             string = self.adjustForRepeats(string)
-            if string.isupper():
+            if string.decode("UTF-8").isupper():
                 voice = settings.voices[settings.UPPERCASE_VOICE]
             else:
                 voice = settings.voices[settings.DEFAULT_VOICE]
@@ -1088,7 +1088,7 @@ class Script(default.Script):
                                 includeContext=False,
                                 priorObj=oldLocusOfFocus)
                             [headerRegions, focusedRegion] = \
-                                         brailleGen.getBrailleRegions(header)
+                                         brailleGen.generateBraille(header)
                             brailleRegions.extend(headerRegions)
                             brailleRegions.append(braille.Region(" "))
 
@@ -1117,7 +1117,7 @@ class Script(default.Script):
                             includeContext=False,
                             priorObj=oldLocusOfFocus)
                         [cellRegions, focusedRegion] = \
-                                           brailleGen.getBrailleRegions(cell)
+                                           brailleGen.generateBraille(cell)
 
                         # Translators: this is the name of the
                         # status column header in the message
@@ -1210,7 +1210,7 @@ class Script(default.Script):
                                                   includeContext=False,
                                                   priorObj=oldLocusOfFocus)
             [brailleRegions, focusedRegion] = \
-                    brailleGen.getBrailleRegions(parent)
+                    brailleGen.generateBraille(parent)
             speech.speak(utterances)
 
             apptExtents = event.source.queryComponent().getExtents(0)
@@ -1229,7 +1229,7 @@ class Script(default.Script):
                                 includeContext=False,
                                 priorObj=oldLocusOfFocus)
                             [apptRegions, focusedRegion] = \
-                                brailleGen.getBrailleRegions(event.source)
+                                brailleGen.generateBraille(event.source)
                             brailleRegions.extend(apptRegions)
                             speech.speak(utterances)
 
@@ -1293,7 +1293,7 @@ class Script(default.Script):
                             includeContext=False,
                             priorObj=oldLocusOfFocus)
                         [apptRegions, focusedRegion] = \
-                            brailleGen.getBrailleRegions(child)
+                            brailleGen.generateBraille(child)
                         brailleRegions.extend(apptRegions)
                         speech.speak(utterances)
 
@@ -1629,6 +1629,24 @@ class Script(default.Script):
         #
         default.Script.onStateChanged(self, event)
 
+    def isActivatableEvent(self, event):
+        """Returns True if the given event is one that should cause this
+        script to become the active script.  This is only a hint to
+        the focus tracking manager and it is not guaranteed this
+        request will be honored.  Note that by the time the focus
+        tracking manager calls this method, it thinks the script
+        should become active.  This is an opportunity for the script
+        to say it shouldn't.
+        """
+
+        # If the Evolution window is not focused, ignore this event.
+        #
+        window = self.getTopLevel(event.source)
+        if window and not window.getState().contains(pyatspi.STATE_ACTIVE):
+            return False
+
+        return True
+
     def onFocus(self, event):
         """Called whenever an object gets focus.
 
