@@ -119,6 +119,7 @@ userCustomizableSettings = [
     "brailleRolenameStyle",
     "brailleSelectorIndicator",
     "brailleLinkIndicator",
+    "brailleAlignmentStyle",
     "enableBrailleMonitor",
     "enableMagnifier",
     "enableMagLiveUpdating",
@@ -172,6 +173,7 @@ userCustomizableSettings = [
     "textAttributesBrailleIndicator",
     "enableProgressBarUpdates",
     "progressBarUpdateInterval",
+    "progressBarVerbosity",
     "enableContractedBraille",
     "brailleContractionTable",
     "enableMouseReview",
@@ -271,6 +273,19 @@ BRAILLE_LINK_8    = 0x80 # 10000000
 BRAILLE_LINK_BOTH = 0xc0 # 11000000
 brailleLinkIndicator = BRAILLE_LINK_BOTH
 
+# Braille alignment styles.  These say how to align text on the
+# edges of the braille display.  The brailleAlignmentMargin value
+# says how close to the edge of the braille the display the cursor
+# cell can get.  The brailleMaximumJump says how far we can jump
+# the display when aligning by word.
+#
+ALIGN_BRAILLE_BY_EDGE   = 0
+ALIGN_BRAILLE_BY_MARGIN = 1
+ALIGN_BRAILLE_BY_WORD   = 2
+brailleAlignmentMargin  = 3
+brailleMaximumJump      = 8
+brailleAlignmentStyle   = ALIGN_BRAILLE_BY_EDGE
+
 # Speech punctuation levels (see verbalizePunctuationStyle).
 #
 PUNCTUATION_STYLE_NONE = 3
@@ -366,6 +381,10 @@ enableBrailleContext    = True
 #
 enableBrailleGrouping   = False
 
+# The timeout (in milliseconds) to use for messages flashed in braille.
+#
+brailleFlashTime        = 5000
+
 # If True, enable braille monitor.
 #
 enableBrailleMonitor    = False
@@ -663,6 +682,18 @@ enableProgressBarUpdates = True
 #
 progressBarUpdateInterval = 10
 
+# When progress bar updates should be spoken, assuming they are enabled.
+# ALL means that any progress bar update will be spoken. APPLICATION
+# means that progress bar updates from the active application will be
+# spoken, even if they are not in the active window. WINDOW means that
+# the progress bar must be contained in the active window in order to
+# be spoken. The default verbosity level is APPLICATION.
+#
+PROGRESS_BAR_ALL = 0
+PROGRESS_BAR_APPLICATION = 1
+PROGRESS_BAR_WINDOW = 2
+progressBarVerbosity = PROGRESS_BAR_APPLICATION
+
 # Whether or not to present the 'read only' attribute of text areas
 # if we can detect they are read only or not.
 #
@@ -1300,6 +1331,18 @@ speechNodeLevelString = _("tree level %d")
 #
 speechNestingLevelString = _("Nesting level %d")
 
+# Translators: this is a indication of the focused icon and the
+# count of the total number of icons within an icon panel. An
+# example of an icon panel is the Nautilus folder view.
+#
+speechIconIndexString = _("on %(index)d of %(total)d")
+
+# Translators: this refers to the position of an item in a list
+# or group of objects, such as menu items in a menu, radio buttons
+# in a radio button group, combobox item in a combobox, etc.
+#
+speechGroupIndexString = _("%(index)d of %(total)d")
+
 # string to indicate end of printed line for braille displays:
 #
 disableBrailleEOL = False
